=begin
#Enterprise Mission Assurance Support Service (eMASS)

#The Enterprise Mission Assurance Support Service (eMASS) Representational State Transfer (REST) Application Programming Interface (API) enables users to perform assessments and complete actions associated with system records.   The `emasser` is a command-line interface (CLI) tool that implements all of the  eMASS endpoints defined in the eMASS REST API v3.2, dated October 21, 2021.</br><br>  <strong>Register CLI</strong></br> New users will need to register an API key with the eMASS development team prior to accessing the site for the first time. The eMASS REST API requires a client certificate (SSL/TLS, DoD PKI only) where {url}/api/register (POST) is used to register the client certificate.</br></br>  Every call to the eMASS REST API will require the use of the agreed upon public key certificate and API key. The API key must be provided in the request header for all endpoint calls (api-key). If the service receives an untrusted certificate or API key, a 401 error response code will be returned along with an error message.</br></br>  <strong>Available Request Headers:</strong></br> <table>   <tr>     <th align=left>key</th>     <th align=left>Example Value</th>     <th align=left>Description</th>   </tr>   <tr>     <td>`api-key`</td>     <td>api-key-provided-by-emass</td>     <td>This API key must be provided in the request header for all endpoint calls</td>   </tr>   <tr>     <td>`user-uid`</td>     <td>USER.UID.KEY</td>     <td>This User unique identifier key must be provided in the request header for all PUT, POST, and DELETE endpoint calls</td>   </tr>   <tr>     <td></td><td></td>     <td>       Note: For DoD users this is the DoD ID Number (EIDIPI) on their DoD CAC     </td>   </tr> </table>  </br><strong>Approve API Client for Actionable Requests</strong></br> Users are required to log-in to eMASS and grant permissions for a client to update data within eMASS on their behalf. This is only required for actionable requests (PUT, POST, DELETE). The Registration Endpoint and all GET requests can be accessed without completing this process with the correct permissions. Please note that leaving a field parameter blank (for PUT/POST requests) has the potential to clear information in the active eMASS records.  To establish an account with eMASS and/or acquire an api-key/user-uid, contact one of the listed POC: 

OpenAPI spec version: v3.2
Contact: disa.meade.id.mbx.emass-tier-iii-support@mail.mil
Generated by: https://github.com/swagger-api/swagger-codegen.git
Swagger Codegen version: 3.0.26
=end

# Common files
require 'emass_client/api_client'
require 'emass_client/api_error'
require 'emass_client/version'
require 'emass_client/configuration'

# Models
require 'emass_client/models/artifacts_delete'
require 'emass_client/models/artifacts_delete_inner'
require 'emass_client/models/artifacts_get'
require 'emass_client/models/artifacts_request_put_body'
require 'emass_client/models/artifacts_response_del'
require 'emass_client/models/artifacts_response_del_data'
require 'emass_client/models/artifacts_response_get'
require 'emass_client/models/artifacts_response_put_post'
require 'emass_client/models/cac_get'
require 'emass_client/models/cac_request_post_body'
require 'emass_client/models/cac_response_get'
require 'emass_client/models/cac_response_post'
require 'emass_client/models/cac_response_post_data'
require 'emass_client/models/cmmc_get'
require 'emass_client/models/cmmc_response_get'
require 'emass_client/models/connectivity_ccsd'
require 'emass_client/models/controls_get'
require 'emass_client/models/controls_put'
require 'emass_client/models/controls_request_put_body'
require 'emass_client/models/controls_response_get'
require 'emass_client/models/controls_response_put'
require 'emass_client/models/definition_transitions'
require 'emass_client/models/delete_milestone'
require 'emass_client/models/delete_poam'
require 'emass_client/models/empty200_response'
require 'emass_client/models/error'
require 'emass_client/models/instances_transitions'
require 'emass_client/models/milestone_response_get'
require 'emass_client/models/milestone_response_post'
require 'emass_client/models/milestone_response_put'
require 'emass_client/models/milestones_delete'
require 'emass_client/models/milestones_get'
require 'emass_client/models/milestones_post'
require 'emass_client/models/milestones_put'
require 'emass_client/models/milestones_put_post_delete'
require 'emass_client/models/milestones_request_post_body'
require 'emass_client/models/milestones_request_put_body'
require 'emass_client/models/milestones_required_post'
require 'emass_client/models/milestones_required_put'
require 'emass_client/models/model_200'
require 'emass_client/models/model_201'
require 'emass_client/models/model_201_meta'
require 'emass_client/models/model_400'
require 'emass_client/models/model_400_meta'
require 'emass_client/models/model_400_response'
require 'emass_client/models/model_401'
require 'emass_client/models/model_401_meta'
require 'emass_client/models/model_403'
require 'emass_client/models/model_403_meta'
require 'emass_client/models/model_404'
require 'emass_client/models/model_404_response'
require 'emass_client/models/model_405'
require 'emass_client/models/model_405_meta'
require 'emass_client/models/model_411'
require 'emass_client/models/model_411_meta'
require 'emass_client/models/model_490'
require 'emass_client/models/model_490_meta'
require 'emass_client/models/model_500'
require 'emass_client/models/model_500_meta'
require 'emass_client/models/pac_get'
require 'emass_client/models/pac_post'
require 'emass_client/models/pac_request_post_body'
require 'emass_client/models/pac_response_get'
require 'emass_client/models/pac_response_post'
require 'emass_client/models/poam_delete'
require 'emass_client/models/poam_get'
require 'emass_client/models/poam_post'
require 'emass_client/models/poam_post_put_del'
require 'emass_client/models/poam_put'
require 'emass_client/models/poam_request_post_body'
require 'emass_client/models/poam_request_put_body'
require 'emass_client/models/poam_required_post'
require 'emass_client/models/poam_required_put'
require 'emass_client/models/poam_response_delete'
require 'emass_client/models/poam_response_get'
require 'emass_client/models/poam_response_post'
require 'emass_client/models/poam_response_put'
require 'emass_client/models/register'
require 'emass_client/models/register_data'
require 'emass_client/models/register_user_request_post_body'
require 'emass_client/models/role_category'
require 'emass_client/models/roles'
require 'emass_client/models/ssps'
require 'emass_client/models/stage'
require 'emass_client/models/static_code_application'
require 'emass_client/models/static_code_request_post_body'
require 'emass_client/models/static_code_required_post'
require 'emass_client/models/static_code_required_post_application'
require 'emass_client/models/success200_response'
require 'emass_client/models/system_id_artifacts_body'
require 'emass_client/models/system_response'
require 'emass_client/models/system_roles_category_response'
require 'emass_client/models/system_roles_response'
require 'emass_client/models/system_roles_response_data'
require 'emass_client/models/systems'
require 'emass_client/models/test'
require 'emass_client/models/test_data'
require 'emass_client/models/test_results_get'
require 'emass_client/models/test_results_post'
require 'emass_client/models/test_results_request_post_body'
require 'emass_client/models/test_results_response_get'
require 'emass_client/models/test_results_response_post'
require 'emass_client/models/users'
require 'emass_client/models/workflow_definition_get'
require 'emass_client/models/workflow_definition_response_get'
require 'emass_client/models/workflow_instances_get'
require 'emass_client/models/workflow_instances_response_get'

# APIs
require 'emass_client/api/artifacts_api'
require 'emass_client/api/artifacts_export_api'
require 'emass_client/api/cac_api'
require 'emass_client/api/cmmc_assessments_api'
require 'emass_client/api/controls_api'
require 'emass_client/api/milestones_api'
require 'emass_client/api/pac_api'
require 'emass_client/api/poam_api'
require 'emass_client/api/registration_api'
require 'emass_client/api/static_code_scans_api'
require 'emass_client/api/system_roles_api'
require 'emass_client/api/systems_api'
require 'emass_client/api/test_api'
require 'emass_client/api/test_results_api'
require 'emass_client/api/workflow_definitions_api'
require 'emass_client/api/workflow_instances_api'

module EmassClient
  class << self
    # Customize default settings for the SDK using block.
    #   EmassClient.configure do |config|
    #     config.username = "xxx"
    #     config.password = "xxx"
    #   end
    # If no block given, return the default Configuration object.
    def configure
      if block_given?
        yield(Configuration.default)
      else
        Configuration.default
      end
    end
  end
end
