=begin
#Enterprise Mission Assurance Support Service (eMASS)

#The Enterprise Mission Assurance Support Service (eMASS) Representational State Transfer (REST) Application Programming Interface (API) enables users to perform assessments and complete actions associated with system records.   The `emasser` is a command-line interface (CLI) tool that implements all of the  eMASS endpoints defined in the eMASS REST API v3.2, dated October 21, 2021.</br><br>  <strong>Register CLI</strong></br> New users will need to register an API key with the eMASS development team prior to accessing the site for the first time. The eMASS REST API requires a client certificate (SSL/TLS, DoD PKI only) where {url}/api/register (POST) is used to register the client certificate.</br></br>  Every call to the eMASS REST API will require the use of the agreed upon public key certificate and API key. The API key must be provided in the request header for all endpoint calls (api-key). If the service receives an untrusted certificate or API key, a 401 error response code will be returned along with an error message.</br></br>  <strong>Available Request Headers:</strong></br> <table>   <tr>     <th align=left>key</th>     <th align=left>Example Value</th>     <th align=left>Description</th>   </tr>   <tr>     <td>`api-key`</td>     <td>api-key-provided-by-emass</td>     <td>This API key must be provided in the request header for all endpoint calls</td>   </tr>   <tr>     <td>`user-uid`</td>     <td>USER.UID.KEY</td>     <td>This User unique identifier key must be provided in the request header for all PUT, POST, and DELETE endpoint calls</td>   </tr>   <tr>     <td></td><td></td>     <td>       Note: For DoD users this is the DoD ID Number (EIDIPI) on their DoD CAC     </td>   </tr> </table>  </br><strong>Approve API Client for Actionable Requests</strong></br> Users are required to log-in to eMASS and grant permissions for a client to update data within eMASS on their behalf. This is only required for actionable requests (PUT, POST, DELETE). The Registration Endpoint and all GET requests can be accessed without completing this process with the correct permissions. Please note that leaving a field parameter blank (for PUT/POST requests) has the potential to clear information in the active eMASS records.  To establish an account with eMASS and/or acquire an api-key/user-uid, contact one of the listed POC: 

OpenAPI spec version: v3.2
Contact: disa.meade.id.mbx.emass-tier-iii-support@mail.mil
Generated by: https://github.com/swagger-api/swagger-codegen.git
Swagger Codegen version: 3.0.26
=end

require 'date'

module EmassClient
  class InstancesTransitions
    # [Read-Only] Comments entered by the user when performing the transition.
    attr_accessor :comments

    # [Read-Only] User that performed the workflow transition.
    attr_accessor :created_by

    # [Read-Only] Date the workflow instance or the workflow transition was created.
    attr_accessor :created_date

    # [Read-Only] Description of the stage transition. This matches the action dropdown that appears for PAC users.
    attr_accessor :description

    # [Read-Only] The landing stage that is active after performing a transition.
    attr_accessor :end_stage

    # [Read-Only] The beginning stage that is active before performing a transition.
    attr_accessor :start_stage

    class EnumAttributeValidator
      attr_reader :datatype
      attr_reader :allowable_values

      def initialize(datatype, allowable_values)
        @allowable_values = allowable_values.map do |value|
          case datatype.to_s
          when /Integer/i
            value.to_i
          when /Float/i
            value.to_f
          else
            value
          end
        end
      end

      def valid?(value)
        !value || allowable_values.include?(value)
      end
    end

    # Attribute mapping from ruby-style variable name to JSON key.
    def self.attribute_map
      {
        :'comments' => :'comments',
        :'created_by' => :'createdBy',
        :'created_date' => :'createdDate',
        :'description' => :'description',
        :'end_stage' => :'endStage',
        :'start_stage' => :'startStage'
      }
    end

    # Attribute type mapping.
    def self.openapi_types
      {
        :'comments' => :'Object',
        :'created_by' => :'Object',
        :'created_date' => :'Object',
        :'description' => :'Object',
        :'end_stage' => :'Object',
        :'start_stage' => :'Object'
      }
    end

    # List of attributes with nullable: true
    def self.openapi_nullable
      Set.new([
      ])
    end
  
    # Initializes the object
    # @param [Hash] attributes Model attributes in the form of hash
    def initialize(attributes = {})
      if (!attributes.is_a?(Hash))
        fail ArgumentError, "The input argument (attributes) must be a hash in `EmassClient::InstancesTransitions` initialize method"
      end

      # check to see if the attribute exists and convert string to symbol for hash key
      attributes = attributes.each_with_object({}) { |(k, v), h|
        if (!self.class.attribute_map.key?(k.to_sym))
          fail ArgumentError, "`#{k}` is not a valid attribute in `EmassClient::InstancesTransitions`. Please check the name to make sure it's valid. List of attributes: " + self.class.attribute_map.keys.inspect
        end
        h[k.to_sym] = v
      }

      if attributes.key?(:'comments')
        self.comments = attributes[:'comments']
      end

      if attributes.key?(:'created_by')
        self.created_by = attributes[:'created_by']
      end

      if attributes.key?(:'created_date')
        self.created_date = attributes[:'created_date']
      end

      if attributes.key?(:'description')
        self.description = attributes[:'description']
      end

      if attributes.key?(:'end_stage')
        self.end_stage = attributes[:'end_stage']
      end

      if attributes.key?(:'start_stage')
        self.start_stage = attributes[:'start_stage']
      end
    end

    # Show invalid properties with the reasons. Usually used together with valid?
    # @return Array for valid properties with the reasons
    def list_invalid_properties
      invalid_properties = Array.new
      invalid_properties
    end

    # Check to see if the all the properties in the model are valid
    # @return true if the model is valid
    def valid?
      comments_validator = EnumAttributeValidator.new('Object', ['Approved the categorization', 'Submitted the categorization', 'Categorized the system as HMM', 'Selected POA&M Items'])
      return false unless comments_validator.valid?(@comments)
      description_validator = EnumAttributeValidator.new('Object', ['Initiate Workflow', 'Approve', 'Submit New Package', 'Disapprove and Move Forward', 'Cancel', 'Deny'])
      return false unless description_validator.valid?(@description)
      end_stage_validator = EnumAttributeValidator.new('Object', ['PM/ISO', 'Echelon I', 'Echelon II', 'Echelon III', 'Categorize System', 'Submit Categorization', 'Approval', 'Complete'])
      return false unless end_stage_validator.valid?(@end_stage)
      start_stage_validator = EnumAttributeValidator.new('Object', ['Not Started', 'PM/ISO', 'Categorize System', 'Submit Categorization', 'Approval', 'Deny'])
      return false unless start_stage_validator.valid?(@start_stage)
      true
    end

    # Custom attribute writer method checking allowed values (enum).
    # @param [Object] comments Object to be assigned
    def comments=(comments)
      validator = EnumAttributeValidator.new('Object', ['Approved the categorization', 'Submitted the categorization', 'Categorized the system as HMM', 'Selected POA&M Items'])
      unless validator.valid?(comments)
        fail ArgumentError, "invalid value for \"comments\", must be one of #{validator.allowable_values}."
      end
      @comments = comments
    end

    # Custom attribute writer method checking allowed values (enum).
    # @param [Object] description Object to be assigned
    def description=(description)
      validator = EnumAttributeValidator.new('Object', ['Initiate Workflow', 'Approve', 'Submit New Package', 'Disapprove and Move Forward', 'Cancel', 'Deny'])
      unless validator.valid?(description)
        fail ArgumentError, "invalid value for \"description\", must be one of #{validator.allowable_values}."
      end
      @description = description
    end

    # Custom attribute writer method checking allowed values (enum).
    # @param [Object] end_stage Object to be assigned
    def end_stage=(end_stage)
      validator = EnumAttributeValidator.new('Object', ['PM/ISO', 'Echelon I', 'Echelon II', 'Echelon III', 'Categorize System', 'Submit Categorization', 'Approval', 'Complete'])
      unless validator.valid?(end_stage)
        fail ArgumentError, "invalid value for \"end_stage\", must be one of #{validator.allowable_values}."
      end
      @end_stage = end_stage
    end

    # Custom attribute writer method checking allowed values (enum).
    # @param [Object] start_stage Object to be assigned
    def start_stage=(start_stage)
      validator = EnumAttributeValidator.new('Object', ['Not Started', 'PM/ISO', 'Categorize System', 'Submit Categorization', 'Approval', 'Deny'])
      unless validator.valid?(start_stage)
        fail ArgumentError, "invalid value for \"start_stage\", must be one of #{validator.allowable_values}."
      end
      @start_stage = start_stage
    end

    # Checks equality by comparing each attribute.
    # @param [Object] Object to be compared
    def ==(o)
      return true if self.equal?(o)
      self.class == o.class &&
          comments == o.comments &&
          created_by == o.created_by &&
          created_date == o.created_date &&
          description == o.description &&
          end_stage == o.end_stage &&
          start_stage == o.start_stage
    end

    # @see the `==` method
    # @param [Object] Object to be compared
    def eql?(o)
      self == o
    end

    # Calculates hash code according to all attributes.
    # @return [Integer] Hash code
    def hash
      [comments, created_by, created_date, description, end_stage, start_stage].hash
    end

    # Builds the object from hash
    # @param [Hash] attributes Model attributes in the form of hash
    # @return [Object] Returns the model itself
    def self.build_from_hash(attributes)
      new.build_from_hash(attributes)
    end

    # Builds the object from hash
    # @param [Hash] attributes Model attributes in the form of hash
    # @return [Object] Returns the model itself
    def build_from_hash(attributes)
      return nil unless attributes.is_a?(Hash)
      self.class.openapi_types.each_pair do |key, type|
        if type =~ /\AArray<(.*)>/i
          # check to ensure the input is an array given that the attribute
          # is documented as an array but the input is not
          if attributes[self.class.attribute_map[key]].is_a?(Array)
            self.send("#{key}=", attributes[self.class.attribute_map[key]].map { |v| _deserialize($1, v) })
          end
        elsif !attributes[self.class.attribute_map[key]].nil?
          self.send("#{key}=", _deserialize(type, attributes[self.class.attribute_map[key]]))
        elsif attributes[self.class.attribute_map[key]].nil? && self.class.openapi_nullable.include?(key)
          self.send("#{key}=", nil)
        end
      end

      self
    end

    # Deserializes the data based on type
    # @param string type Data type
    # @param string value Value to be deserialized
    # @return [Object] Deserialized data
    def _deserialize(type, value)
      case type.to_sym
      when :DateTime
        DateTime.parse(value)
      when :Date
        Date.parse(value)
      when :String
        value.to_s
      when :Integer
        value.to_i
      when :Float
        value.to_f
      when :Boolean
        if value.to_s =~ /\A(true|t|yes|y|1)\z/i
          true
        else
          false
        end
      when :Object
        # generic object (usually a Hash), return directly
        value
      when /\AArray<(?<inner_type>.+)>\z/
        inner_type = Regexp.last_match[:inner_type]
        value.map { |v| _deserialize(inner_type, v) }
      when /\AHash<(?<k_type>.+?), (?<v_type>.+)>\z/
        k_type = Regexp.last_match[:k_type]
        v_type = Regexp.last_match[:v_type]
        {}.tap do |hash|
          value.each do |k, v|
            hash[_deserialize(k_type, k)] = _deserialize(v_type, v)
          end
        end
      else # model
        EmassClient.const_get(type).build_from_hash(value)
      end
    end

    # Returns the string representation of the object
    # @return [String] String presentation of the object
    def to_s
      to_hash.to_s
    end

    # to_body is an alias to to_hash (backward compatibility)
    # @return [Hash] Returns the object in the form of hash
    def to_body
      to_hash
    end

    # Returns the object in the form of hash
    # @return [Hash] Returns the object in the form of hash
    def to_hash
      hash = {}
      self.class.attribute_map.each_pair do |attr, param|
        value = self.send(attr)
        if value.nil?
          is_nullable = self.class.openapi_nullable.include?(attr)
          next if !is_nullable || (is_nullable && !instance_variable_defined?(:"@#{attr}"))
        end

        hash[param] = _to_hash(value)
      end
      hash
    end

    # Outputs non-array value in the form of hash
    # For object, use to_hash. Otherwise, just return the value
    # @param [Object] value Any valid value
    # @return [Hash] Returns the value in the form of hash
    def _to_hash(value)
      if value.is_a?(Array)
        value.compact.map { |v| _to_hash(v) }
      elsif value.is_a?(Hash)
        {}.tap do |hash|
          value.each { |k, v| hash[k] = _to_hash(v) }
        end
      elsif value.respond_to? :to_hash
        value.to_hash
      else
        value
      end
    end  end
end
